@@ -11,7 +11,7 @@ use POSIX qw( strftime );
 use Moo;
 with "Archive::BagIt::Role::Portability";
 
-our $VERSION = '0.096'; # VERSION
+our $VERSION = '0.097'; # VERSION
 
 # ABSTRACT: The main module to handle bags.
 
@@ -311,14 +311,24 @@ sub _find_baginfo_idx {
 ###############################################
 
 
-sub verify_baginfo {
-    my ($self) = @_;
+sub _verify_baginfo {
+    my ($self, $info) = @_;
     my %keys;
-    my $info = $self->bag_info();
     my $ret = 1;
     if (defined $info) {
-        foreach my $entry (@{$self->bag_info()}) {
+        foreach my $entry (@{$info}) {
             my ($key, $value) = %{$entry};
+            my $res = _check_key($key); # check key
+            if ($res ne "") {
+                push @{$self->{errors}}, $res;
+                $ret = undef;
+            }
+            $res = _check_value($value); # check value
+            if ($res ne "") {
+                push @{$self->{errors}}, $res;
+                $ret = undef;
+            }
+            # code to prepare check of uniqueness
             if ($self->is_baginfo_key_reserved($key)) {
                 $keys{ lc $key }++;
             }
@@ -326,6 +336,7 @@ sub verify_baginfo {
                 $keys{ $key }++
             }
         }
+        # check for uniqueness
         foreach my $key (keys %keys) {
             if ($self->is_baginfo_key_reserved_as_uniq($key)) {
                 if ($keys{$key} > 1) {
@@ -334,21 +345,36 @@ sub verify_baginfo {
                 }
             }
         }
-    }
-    # check for payload oxum
-    my ($loaded_payloadoxum) = $self->get_baginfo_values_by_key('Payload-Oxum');
-    if (defined $loaded_payloadoxum) {
-        my ($octets, $streamcount) = $self->calc_payload_oxum();
-        if ("$octets.$streamcount" ne $loaded_payloadoxum) {
-            push @{$self->{errors}}, "Payload-Oxum differs, calculated $octets.$streamcount but $loaded_payloadoxum was expected by bag-info.txt";
-            $ret = undef;
+
+        # check for payload oxum
+        my ($loaded_payloadoxum) = $self->get_baginfo_values_by_key('Payload-Oxum');
+        if (defined $loaded_payloadoxum) {
+            my ($octets, $streamcount) = $self->calc_payload_oxum();
+            if ("$octets.$streamcount" ne $loaded_payloadoxum) {
+                push @{$self->{errors}}, "Payload-Oxum differs, calculated $octets.$streamcount but $loaded_payloadoxum was expected by bag-info.txt";
+                $ret = undef;
+            }
+        }
+        else {
+            push @{$self->{warnings}}, "Payload-Oxum was expected in bag-info.txt, but not found!"; # payload-oxum is recommended, but optional
         }
     } else {
-        push @{$self->{warnings}}, "Payload-Oxum was expected in bag-info.txt, but not found!"; # payload-oxum is recommended, but optional
+        if (exists $self->{bag_info_file}) {
+            push @{$self->{errors}}, "'bag-info.txt' exists, but is not (partially) parseable!";
+        }
     }
     return $ret;
 }
 
+sub verify_baginfo {
+    my ($self) = @_;
+    my $info = $self->bag_info();
+    if (List::Util::any { /the baginfo file .* could not be parsed correctly/ } @{$self->{'errors'}}) {
+        return;
+    }
+    return $self->_verify_baginfo($info);
+}
+
 ###############################################
 
 
@@ -383,18 +409,56 @@ sub _replace_baginfo_by_first_match {
 
 ###############################################
 
+sub _check_key {
+    my ($key) = @_;
+    if (!defined $key) {
+        return "key should match '[^\\r\\n:]+', but is not defined";
+    }
+    if ($key =~ m/[\r\n]/s) {
+        return "key should match '[^\\r\\n:]+', but contains newlines (key='$key')";
+    }
+    if ($key =~ m/:/) {
+        return "key should not contain a colon! (key='$key')";
+    }
+    if ($key =~ m/^$/) {
+        return "key should have a length > null (key='')";
+    }
+    return "";
+}
+
+###############################################
+
+sub _check_key_or_croak {
+    my ($key) = @_;
+    my $res = _check_key($key);
+    if ($res eq "") { return 1;}
+    croak $res;
+}
+
+###############################################
+
+sub _check_value {
+    my ($value) = @_;
+    if (!defined $value) { return "value should match '[^\\r\\n:]+', but is not defined"; }
+    if ($value =~ m/^$/s) { return "value should have a length > null (value='')"; }
+    return "";
+}
+
+###############################################
 
 sub append_baginfo_by_key {
     my ($self, $searchkey, $newvalue) = @_;
-    if (defined $searchkey) {
-        if (-1 < index($searchkey, ":")) { croak "key should not contain a colon! (searchkey='$searchkey')"; }
-        if ($self->is_baginfo_key_reserved_as_uniq($searchkey)) {
-            if (defined $self->get_baginfo_values_by_key($searchkey)) {
-                # hmm, search key is marked as uniq and still exists
-                return;
+    if (_check_key_or_croak($searchkey)) {
+        if (defined $newvalue) {
+            if (-1 < index($searchkey, ":")) {croak "key should not contain a colon! (searchkey='$searchkey')";}
+            if ($self->is_baginfo_key_reserved_as_uniq($searchkey)) {
+                if (defined $self->get_baginfo_values_by_key($searchkey)) {
+                    # hmm, search key is marked as uniq and still exists
+                    return;
+                }
             }
+            push @{$self->{bag_info}}, { $searchkey => $newvalue };
         }
-        push @{$self->{bag_info}}, {$searchkey => $newvalue};
     }
     return 1;
 }
@@ -404,7 +468,7 @@ sub append_baginfo_by_key {
 
 sub add_or_replace_baginfo_by_key {
     my ($self, $searchkey, $newvalue) = @_;
-    if (defined $searchkey) {
+    if ( _check_key_or_croak($searchkey) ) {
         if (-1 < index($searchkey, ":")) { croak "key should not contain a colon! (searchkey='$searchkey')"; }
         if (defined $self->{bag_info}) {
             my $idx = $self->_replace_baginfo_by_first_match( $searchkey, $newvalue);
@@ -722,6 +786,9 @@ sub _parse_bag_info { # parses a bag-info textblob
             push @labels, { $label => $value };
         }
     }
+    if ($textblob ne "") {
+        push @{$self->{errors}}, "the baginfo file '".$self->{bag_info_file}."' could not be parsed correctly, because following residue not fullfill the match requirements: '$textblob'";
+    }
     # The RFC does not allow reordering:
     #my @sorted = __sort_bag_info(@labels);
     #return \@sorted;
@@ -740,6 +807,7 @@ sub _build_bag_info {
         }
         close($BAGINFO);
         my $lines = join("", @lines);
+        $self->{bag_info_file}=$file;
         return $self->_parse_bag_info($lines);
     }
     # bag-info.txt is optional
@@ -992,7 +1060,7 @@ Archive::BagIt - The main module to handle bags.
 
 =head1 VERSION
 
-version 0.096
+version 0.097
 
 =head1 NAME
 
